@@ -37,7 +37,7 @@ func (b jsonBool) hashCode(_ *options) [8]byte {
 }
 
 func (b jsonBool) Diff(n JsonNode, opts ...Option) Diff {
-	o := refine(&options{retain: opts}, nil)
+	o := refine(newOptions(opts), nil)
 	strategy := getPatchStrategy(o)
 	return b.diff(n, make(Path, 0), o, strategy)
 }
@@ -48,7 +48,10 @@ func (b jsonBool) diff(
 	opts *options,
 	strategy patchStrategy,
 ) Diff {
-	return diff(b, n, path, opts, strategy)
+	// Use event-driven diff architecture
+	events := generateSimpleEvents(b, n, opts)
+	processor := newSimpleDiffProcessor(path, opts, strategy)
+	return processor.ProcessEvents(events)
 }
 
 func (b jsonBool) Patch(d Diff) (JsonNode, error) {
