@@ -93,169 +93,6 @@ func (a jsonList) diffEventDriven(b jsonList, path Path, opts *options, strategy
 	return processor.ProcessEvents(events)
 }
 
-func (a jsonList) diffRest(
-	pathIndex PathIndex,
-	b jsonList,
-	path Path,
-	aHashes, bHashes, commonSequence []interface{},
-	previous JsonNode,
-	opts *options,
-	strategy patchStrategy,
-) Diff {
-	var aCursor, bCursor, commonSequenceCursor int
-	pathCursor := pathIndex
-	pathNow := func() Path {
-		return append(path.clone().drop(), pathCursor)
-	}
-	endA := func() bool {
-		return aCursor == len(a)
-	}
-	endB := func() bool {
-		return bCursor == len(b)
-	}
-	atCommonA := func() bool {
-		if endA() || len(commonSequence) == 0 {
-			return false
-		}
-		return aHashes[aCursor] == commonSequence[0]
-	}
-	atCommonB := func() bool {
-		if endB() || len(commonSequence) == 0 {
-			return false
-		}
-		return bHashes[bCursor] == commonSequence[0]
-	}
-	d := Diff{{
-		Path:   pathNow(),
-		Before: []JsonNode{previous},
-	}}
-	haveDiff := func() bool {
-		if len(d) == 0 {
-			return false
-		}
-		if len(d[0].Add) > 0 || len(d[0].Remove) > 0 {
-			return true
-		}
-		return false
-	}
-	after := func() []JsonNode {
-		i := aCursor - commonSequenceCursor
-		if i+1 > len(a) {
-			return []JsonNode{voidNode{}}
-		}
-		return []JsonNode{a[i]}
-	}
-
-accumulatingDiff:
-	for {
-		switch {
-		case endA():
-			// We are at the end of A so there are no more
-			// common elements. So we accumulate the rest
-			// of B as additions. The path cursor advances
-			// by 2 because the result is getting longer
-			// by 1 and we are moving to the next element.
-			for !endB() {
-				d[0].Add = append(d[0].Add, b[bCursor])
-				bCursor++
-				pathCursor += 2
-			}
-			break accumulatingDiff
-		case endB():
-			// We are at the end of B so there are no more
-			// common elements. So we accumulate the rest
-			// of A as removals. The path cursor stays the
-			// same because the result is getting shorter
-			// by 1 but we are also moving to the next
-			// element.
-			for !endA() {
-				d[0].Remove = append(d[0].Remove, a[aCursor])
-				aCursor++
-			}
-			break accumulatingDiff
-		case atCommonA() && atCommonB():
-			// We are at a common element of A and B.
-			// All cursors advance because we are moving
-			// past a common element.
-			aCursor++
-			bCursor++
-			commonSequenceCursor++
-			pathCursor++
-			break accumulatingDiff
-		case atCommonA():
-			// We are at a common element in A. We need to
-			// catch up B. Add elements of B until we do.
-			for !atCommonB() {
-				d[0].Add = append(d[0].Add, b[bCursor])
-				bCursor++
-				pathCursor++
-			}
-		case atCommonB():
-			// We are at a common element in B. We need to
-			// catch up A. Remove elements of A until we
-			// do.
-			for !atCommonA() {
-				d[0].Remove = append(d[0].Remove, a[aCursor])
-				aCursor++
-			}
-		case sameContainerType(a[aCursor], b[bCursor], opts):
-			// We are at compatible containers which
-			// contain additional differences. If we've
-			// accumulated differences at this level then
-			// keep them before the sub-diff.
-			subDiff := a[aCursor].diff(b[bCursor], pathNow(), opts, strategy)
-			if haveDiff() {
-				d[0].After = after()
-				d = append(d, subDiff...)
-			} else {
-				d = subDiff
-			}
-			aCursor++
-			bCursor++
-			pathCursor++
-			break accumulatingDiff
-		default:
-			// We are at elements of A and B which are
-			// different. Add them to the accumulated diff
-			// and continue.
-			d[0].Remove = append(d[0].Remove, a[aCursor])
-			d[0].Add = append(d[0].Add, b[bCursor])
-			aCursor++
-			bCursor++
-			pathCursor++
-		}
-	}
-
-	if !haveDiff() {
-		// Throw away temporary diff because we didn't
-		// accumulate anything.
-		d = Diff{}
-	} else {
-		if len(d[0].Path) > len(path) {
-			// This is a subdiff. Don't touch it.
-		} else {
-			// Record context of accumulated diff. If we appended
-			// a sub-diff then it already has context.
-			if len(d) < 2 {
-				d[0].After = after()
-			}
-		}
-	}
-	if endA() && endB() {
-		return d
-	}
-	// Cursors point to the next elements.
-	return append(d, a[aCursor:].diffRest(
-		pathCursor,
-		b[bCursor:],
-		pathNow(),
-		aHashes[aCursor:], bHashes[bCursor:], commonSequence[commonSequenceCursor:],
-		b[bCursor-1],
-		opts,
-		strategy,
-	)...)
-}
-
 func (a jsonList) diffDifferentTypes(n JsonNode, path Path, strategy patchStrategy) Diff {
 	var e DiffElement
 	switch strategy {
@@ -779,56 +616,6 @@ func (cm *contextManager) SetCursors(aIndex, bIndex int) {
 	cm.bIndex = bIndex
 }
 
-func (cm *contextManager) GetBeforeContext(previous JsonNode) []JsonNode {
-	cm.debugLog("Getting before context with previous: %s", previous.Json())
-	return []JsonNode{previous}
-}
-
-func (cm *contextManager) GetAfterContext() []JsonNode {
-	// Calculate After context similar to original diffRest logic
-	// This is the element that comes after the current operation
-
-	if cm.aIndex >= len(cm.originalA) {
-		cm.debugLog("After context: void (A cursor beyond end)")
-		return []JsonNode{voidNode{}}
-	}
-
-	afterElement := cm.originalA[cm.aIndex]
-	cm.debugLog("After context: A[%d] = %s", cm.aIndex, afterElement.Json())
-	return []JsonNode{afterElement}
-}
-
-func (cm *contextManager) AdvanceForMatch() {
-	cm.debugLog("Advancing context for match: A=%d->%d, B=%d->%d",
-		cm.aIndex, cm.aIndex+1, cm.bIndex, cm.bIndex+1)
-	cm.aIndex++
-	cm.bIndex++
-}
-
-func (cm *contextManager) AdvanceForAdd() {
-	cm.debugLog("Advancing context for add: B=%d->%d", cm.bIndex, cm.bIndex+1)
-	cm.bIndex++
-}
-
-func (cm *contextManager) AdvanceForRemove() {
-	cm.debugLog("Advancing context for remove: A=%d->%d", cm.aIndex, cm.aIndex+1)
-	cm.aIndex++
-}
-
-func (cm *contextManager) AdvanceForReplace() {
-	cm.debugLog("Advancing context for replace: A=%d->%d, B=%d->%d",
-		cm.aIndex, cm.aIndex+1, cm.bIndex, cm.bIndex+1)
-	cm.aIndex++
-	cm.bIndex++
-}
-
-func (cm *contextManager) AdvanceForContainer() {
-	cm.debugLog("Advancing context for container: A=%d->%d, B=%d->%d",
-		cm.aIndex, cm.aIndex+1, cm.bIndex, cm.bIndex+1)
-	cm.aIndex++
-	cm.bIndex++
-}
-
 // hasCommonElements quickly checks if two lists share any common elements
 func hasCommonElements(a, b jsonList, opts *options) bool {
 	if len(a) == 0 || len(b) == 0 {
@@ -959,25 +746,6 @@ func myersDiff(a, b jsonList, opts *options) []myersEdit {
 	N := len(a)
 	M := len(b)
 
-	// Handle trivial cases
-	if N == 0 && M == 0 {
-		return []myersEdit{}
-	}
-	if N == 0 {
-		edits := make([]myersEdit, M)
-		for i := 0; i < M; i++ {
-			edits[i] = myersEdit{Type: "insert", BIdx: i, Value: b[i]}
-		}
-		return edits
-	}
-	if M == 0 {
-		edits := make([]myersEdit, N)
-		for i := 0; i < N; i++ {
-			edits[i] = myersEdit{Type: "delete", AIdx: i, Value: a[i]}
-		}
-		return edits
-	}
-
 	// Myers' algorithm implementation
 	MAX := N + M
 	V := make([]int, 2*MAX+1)
@@ -1030,7 +798,7 @@ func buildMyersEdits(a, b jsonList, trace [][]int, opts *options) []myersEdit {
 
 	// Backtrack through the trace
 	for D := len(trace) - 1; D > 0; D-- {
-		prevV := trace[D-1]
+		prevV := trace[D]
 
 		k := x - y
 
