@@ -3,7 +3,7 @@ package jd
 import (
 	"encoding/json"
 
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v3"
 )
 
 func renderJson(i interface{}) string {
