@@ -38,6 +38,22 @@ spec:
 	}
 }
 
+func TestIssue112(t *testing.T) {
+	// https://github.com/josephburnett/jd/issues/112
+	// Arrays with more than 10 elements triggered the Myers diff
+	// algorithm which had an off-by-one in its backtracking logic.
+	ctx := newTestContext(t)
+	checkDiff(ctx,
+		`{"key1":["v01","v02","v03","v04","v05","v06","v07","v08","v09","v10","v11"]}`,
+		`{"key1":["v01","v02","v03","v04","v05","v06","v07","v08","v09","v10","v11 "]}`,
+		`@ ["key1",10]`,
+		`  "v10"`,
+		`- "v11"`,
+		`+ "v11 "`,
+		`]`,
+	)
+}
+
 func TestDebug(t *testing.T) {
 	fuzz(t, `0`, ``, 0)
 }
